@@ -3,32 +3,14 @@ try:
 except ImportError:
     import json
 
-import os
-import sqlite3
 import time
 from collections import UserDict
+from typing import Any
 
 from telegrambotclient.utils import pretty_format
 
 
 class TelegramStorage:
-    def get_field(self, key: str, field: str, expires: int):
-        raise NotImplementedError()
-
-    def update_fields(self, key: str, mapping, expires: int) -> bool:
-        raise NotImplementedError()
-
-    def delete_fields(self, key: str, *field, expires: int) -> bool:
-        raise NotImplementedError()
-
-    def delete_key(self, key: str) -> bool:
-        raise NotImplementedError()
-
-    def data(self, key: str, expires: int):
-        raise NotImplementedError()
-
-
-class MemoryStorage(TelegramStorage):
     __slots__ = ("_data", )
 
     def __init__(self):
@@ -51,8 +33,7 @@ class MemoryStorage(TelegramStorage):
         current_time = int(time.time())
         if session_data.get("_expires", 0) < current_time:
             return self.delete_key(key)
-        else:
-            session_data["_expires"] = current_time + expires
+        session_data["_expires"] = current_time + expires
         for field in fields:
             if field in session_data:
                 del session_data[field]
@@ -77,28 +58,16 @@ class MemoryStorage(TelegramStorage):
 class SQLiteStorage(TelegramStorage):
     __slots__ = ("_db_conn", )
 
-    def __init__(self, db_file: str = None):
-        if db_file is None:
-            self._db_conn = sqlite3.connect(
-                "file:memory?cache=shared&mode=memory", uri=True)
-        else:
-            db_path = os.path.dirname(db_file)
-            if not os.path.exists(db_path):
-                os.mkdir(db_path)
-            self._db_conn = sqlite3.connect(db_file)
-        self._db_conn.row_factory = sqlite3.Row
-        with self._db_conn:
-            self._db_conn.execute("""
-                CREATE TABLE IF NOT EXISTS `t_session` (
-                    `key`        TEXT NOT NULL UNIQUE,
-                    `data`       TEXT NOT NULL,
-                    `expires`    INTEGER NOT NULL,
-                    PRIMARY KEY(`key`)
-                    )
-                """)
-
-    def __del__(self):
-        self._db_conn.close()
+    def __init__(self, db_conn):
+        db_conn.execute("""
+            CREATE TABLE IF NOT EXISTS `t_session` (
+                `key`        TEXT NOT NULL UNIQUE,
+                `data`       TEXT NOT NULL,
+                `expires`    INTEGER NOT NULL,
+                PRIMARY KEY(`key`)
+                )
+            """)
+        self._db_conn = db_conn
 
     def get_field(self, key: str, field: str, expires):
         with self._db_conn:
@@ -107,12 +76,12 @@ class SQLiteStorage(TelegramStorage):
             row_data = cur.fetchone()
             if row_data:
                 current_time = int(time.time())
-                if row_data["expires"] < current_time:
+                if row_data[1] < current_time:
                     return None
                 self._db_conn.execute(
                     "UPDATE t_session SET expires=? WHERE key=?",
                     (current_time + expires, key))
-                return json.loads(row_data["data"]).get(field, None)
+                return json.loads(row_data[0]).get(field, None)
             return None
 
     def update_fields(self, key: str, mapping, expires: int) -> bool:
@@ -122,15 +91,15 @@ class SQLiteStorage(TelegramStorage):
             row_data = cur.fetchone()
             current_time = int(time.time())
             if row_data:
-                data = json.loads(row_data["data"])
-                if row_data["expires"] >= current_time:
+                data = json.loads(row_data[0])
+                if row_data[1] >= current_time:
                     data.update(mapping)
                 else:
                     data = mapping
                 cur = self._db_conn.execute(
                     "UPDATE t_session SET data=?, expires=? WHERE key=?",
                     (json.dumps(data), current_time + expires, key))
-                return bool(cur.rowcount)
+                return cur.rowcount > 0
 
             cur = self._db_conn.execute(
                 """
@@ -140,7 +109,7 @@ class SQLiteStorage(TelegramStorage):
                     expires
                 ) VALUES (?, ?, ?)
                 """, (key, json.dumps(mapping), current_time + expires))
-            return bool(cur.lastrowid)
+            return cur.lastrowid >= 0
 
     def delete_fields(self, key: str, *fields, expires: int) -> bool:
         with self._db_conn:
@@ -149,22 +118,23 @@ class SQLiteStorage(TelegramStorage):
             row_data = cur.fetchone()
             if row_data:
                 current_time = int(time.time())
-                if row_data["expires"] < current_time:
+                if row_data[1] < current_time:
                     return False
-                data = json.loads(row_data["data"])
+                data = json.loads(row_data[0])
                 for field in fields:
                     if field in data:
                         del data[field]
                 cur = self._db_conn.execute(
                     "UPDATE t_session SET data=?, expires=? WHERE key=?",
                     (json.dumps(data), current_time + expires, key))
-                return bool(cur.rowcount)
+                return cur.rowcount > 0
             return False
 
-    def delete_key(self, key: str):
+    def delete_key(self, key: str) -> bool:
         with self._db_conn:
-            self._db_conn.execute("DELETE from t_session WHERE key=? ",
-                                  (key, ))
+            cur = self._db_conn.execute("DELETE from t_session WHERE key=? ",
+                                        (key, ))
+            return cur.rowcount > 0
 
     def data(self, key: str, expires: int):
         cur = self._db_conn.execute(
@@ -172,13 +142,16 @@ class SQLiteStorage(TelegramStorage):
         row_data = cur.fetchone()
         if row_data:
             current_time = int(time.time())
-            if row_data["expires"] < current_time:
+            if row_data[1] < current_time:
                 return {}
             self._db_conn.execute("UPDATE t_session SET expires=? WHERE key=?",
                                   (current_time + expires, key))
-            return json.loads(row_data["data"])
+            return json.loads(row_data[0])
         return {}
 
+    def __del__(self):
+        self._db_conn.close()
+
 
 class RedisStorage(TelegramStorage):
     __slots__ = ("_redis", )
@@ -227,8 +200,8 @@ class RedisStorage(TelegramStorage):
 class MongoDBStorage(TelegramStorage):
     __slots__ = ("_session", )
 
-    def __init__(self, db):
-        self._session = db.session
+    def __init__(self, collection):
+        self._session = collection
 
     def get_field(self, key: str, field: str, expires: int):
         result = None
@@ -258,10 +231,11 @@ class MongoDBStorage(TelegramStorage):
                 }
             }, {"$set": mapping})
         if result.modified_count == 0:
-            result = self._session.update_one({"_id": key}, {"$set": mapping},
-                                              upsert=True)
+            result = self._session.replace_one({"_id": key},
+                                               mapping,
+                                               upsert=True)
 
-        return result.modified_count > 0 or result.upserted_id != None
+        return result.modified_count > 0 or result.upserted_id is not None
 
     def delete_fields(self, key: str, *fields, expires: int) -> bool:
         current_time = int(time.time())
@@ -292,11 +266,10 @@ class MongoDBStorage(TelegramStorage):
                 }
             }, {"$set": {
                 "_expires": current_time + expires
-            }},
-            projection={
-                "_id": 0,
-                "_expires": 0
-            }) or {}
+            }}) or {}
+
+    def __del__(self):
+        self._session.database.client.close()
 
 
 class TelegramSession(UserDict):
@@ -311,14 +284,14 @@ class TelegramSession(UserDict):
         self.id = session_id
         self.expires = expires if expires > 0 else 1800
 
-    def __getitem__(self, field: str):
+    def __getitem__(self, field: str) -> Any:
         value = self.data.get(field, None)
         if value is None:
             value = self._storage.get_field(self.id, field, self.expires)
             self.data[field] = value
         return value
 
-    def get(self, field: str, default=None):
+    def get(self, field: str, default=None) -> Any:
         return self[field] or default
 
     def delete(self, *fields) -> bool:
@@ -332,7 +305,13 @@ class TelegramSession(UserDict):
     def __delitem__(self, field):
         self.delete(field)
 
-    def pop(self, field: str, default=None):
+    def __contains__(self, field: str) -> bool:
+        return super().__contains__(field) or bool(self[field])
+
+    def __repr__(self):
+        return pretty_format(self._data)
+
+    def pop(self, field: str, default=None) -> Any:
         value = self[field] or default
         del self[field]
         return value
@@ -346,5 +325,6 @@ class TelegramSession(UserDict):
         self.data = {}
         return self._storage.delete_key(self.id)
 
-    def __repr__(self):
-        return pretty_format(self._storage.data(self.id, self.expires))
+    @property
+    def _data(self):
+        return self._storage.data(self.id, self.expires)
